-- queenboard.e
--
-- Ben Lin 5/98
--
-- For solving the eight (actually, any number) queens problem.

class QUEENBOARD
	-- blackboard for solving the n queens problem
inherit BLACKBOARD
redefine display
creation make
feature
	size : INTEGER
		-- size of the board = number of queens to place
	board : ARRAY2[INTEGER]
		-- `size' x `size' array of integers represents the board
		-- -1 = "bad" mark (no queen may go here)
		--  0 = empty square
		--  1 = queen
	queen_count : INTEGER
		-- number of queens placed so far
	queen_loc : ARRAY2[INTEGER]
		-- locations of the queens placed so far

	add_observer : ARRAY2[OBSERVER_LIST]
		-- for every square, there is a list of observers that get
		-- notified every time a queen is placed there.
	mark_observer : like add_observer
		-- for every square, there is a list of observers that get
		-- notified every time it is marked bad.

	make(board_size : INTEGER) is
		-- make a n queens blackboard of size `board_size'
	local
		i : INTEGER
		j : INTEGER
		rf : ROW_FILLER
		cf : COL_FILLER
		df : DIAG_FILLER
		rfull : ROW_FULL
		cfull : COL_FULL
		obs_list : OBSERVER_LIST
	do
		!!command_history.make
		size := board_size
		!!board.make(1, size, 1, size)
		!!queen_loc.make(1, size * size, 1, 2)
		!!add_observer.make(1, size, 1, size)
		!!mark_observer.make(1, size, 1, size)
		
		from i := 1
		until i > size
		loop
			from j := 1
			until j > size
			loop
				!!rf.make(i)
				!!cf.make(j)
				!!df.make(i, j)
				!!obs_list.make
				obs_list.add_last(rf)
				obs_list.add_last(cf)
				obs_list.add_last(df)
				add_observer.put(obs_list, i, j)
				!!obs_list.make
				!!rfull.make(i)
				!!cfull.make(j)
				obs_list.add_last(rfull)
				obs_list.add_last(cfull)
				mark_observer.put(obs_list, i, j)
				j := j + 1
			end
			i := i + 1
		end
		queen_count := 0
	end

	finished : BOOLEAN is
		-- finished if `size' queens have been placed and the blackboard
		-- is still alive
	do
		Result := queen_count = size and alive
	end

	get_open : like command_history is
		-- returns a list of all possible `add_queen' commands, i.e.
		-- an `add_queen' on every empty square
	local
		aq : ADD_QUEEN
		i, j : INTEGER
	do
		!!Result.make
		if alive then
			from i := 1
			until i > size
			loop
				from j := 1
				until j > size
				loop
					if board.item(i, j) = 0 then
						!!aq.make(Current, i, j)
						Result.add_last(aq)
					end
					j := j + 1
				end
				i := i + 1
			end
		end

	end

	add_queen(x, y : INTEGER) is
		-- place a queen on (x, y)
	require
		board.item(x, y) = 0
	do
		board.put(1, x, y)
		queen_count := queen_count + 1
		queen_loc.put(x, queen_count, 1)
		queen_loc.put(y, queen_count, 2)
	end

	mark_bad(x, y : INTEGER) is
		-- mark the square (x, y)
	require
		board.item(x, y) = 0
	do
		board.put(-1, x, y)
	end

	remove_queen(x, y : INTEGER) is
		-- remove the queen on (x, y)
	require
		board.item(x, y) = 1
	do
		board.put(0, x, y)
		queen_count := queen_count - 1
	end

	unmark_bad(x, y : INTEGER) is
		-- remove the mark on (x, y)
	require
		board.item(x, y) = -1
	do
		board.put(0, x, y)
	end

	display is
		-- show the chessboard, with queens and marks
	local
		i, j : integer
	do
		from i := 1
		until i > size
		loop
			from j := 1
			until j > size
			loop
				inspect board.item(i, j)
				when 1 then
					io.put_character('Q')
				when 0 then
					io.put_character('.')
				when -1 then
					io.put_character('x')
				else
					io.put_character('?')
				end
				j := j + 1
			end

			io.put_character('%N')
			i := i + 1
		end
		
	end

end 

