-- blackboard.e
--
-- Ben Lin 4/98

-- 9/6/98 - Added comments(!), changed death back to a boolean
--
-- This holds the current state of the solution. For a typical problem,
-- it starts off empty, and as the search progresses, a solution takes
-- form on it as bits and pieces are added. (An image of a group of
-- scientists surrounding a blackboard comes to mind.)

deferred class BLACKBOARD
	-- Holds the current state of the solution

feature
	get_open : COMMAND_LIST is
		-- return a list of possible moves
	deferred end

	finished : BOOLEAN is
		-- true if a solution has been found
	deferred end

	display is
		-- display the blackboard, should be redefined
		-- useful for debugging
	do
		io.put_string("Blackboard%N")
	end

	command_history : COMMAND_LIST
		-- list of commands applied to blackboard so far

	display_command_history is
		-- show the command history
	local
		i : integer
	do
		io.put_string("Command history:%N")
		from i := command_history.lower
		until i > command_history.upper
		loop
			command_history.item(i).display
			i := i + 1
		end
	end

	alive : BOOLEAN is
		-- True if the blackboard is alive
	do
		Result := not death
	end

	dead : BOOLEAN is
		-- True if the blackboard is dead
	do
		Result := death
	end

feature {COMMAND}
	suicide is
		-- Kill the blackboard
	require
		not death
	do
		death := true
	end

	rebirth is
		-- Make the blackboard live again.
	require
		death
	do
		death := false
	end

feature {NONE}
	death : BOOLEAN

end
