-- Logic problem - "39. Method For Its Make"
-- from "Logic Problems" No. 31 (by A.J. Duncum)
-- Ben Lin 30/5/98
--
-- See tricky.e for another example
--
-- This version was written to test multicommands (composite commands)

class MULTITEST
creation make
feature
	pb : LP_BOARD
	size : INTEGER is 5
	sub_count : INTEGER is 4

	-- The 4 categories are:
	-- 1 = Action (Align, Fold, Insert, Slide, Stick)
	-- 2 = Piece (Corner, Edge, Flag, Strip, Tab)
	-- 3 = Name (A, B, C, D, E)
	-- 4 = Destination (F, G, H, I, J)

	--ss : DFS_SEARCH
	ss : I_SEARCH

	multitick : MULTICOMMAND

	make is
	do
		!!pb.make(sub_count, size)
		!!multitick.make(pb)
		setup_clues
		!!ss
		ss.doit(pb)
		if not pb.finished then
			io.put_string("No solution! :(%N")
		else
			io.put_string("Solution found:%N")
			pb.display
		end
	end

	setup_clues is
	do
		-- Observers must come first

		larger_than(4, 1, 1, 4, 3)
			-- 4. The letter of the piece on F comes after the letter
			-- of the piece to be slid

		-- Ticks and crosses

		tick(1, 3, 3, 1)
			-- 1. Piece A is inserted

		cross(2, 3, 3, 2)
			-- 2. Piece B is not a flap

		cross(1, 2, 2, 3)
			-- 2. The flap is not folded

		tick(2, 3, 4, 4)
			-- 3. Piece D is a strip

		tick(3, 4, 5, 5)
			-- 3. ...E connects with J

		cross(2, 3, 1, 5)
			-- 3. ...E is not the corner

		tick(1, 2, 1, 1)
			-- 3. ...Corner is to be aligned

		tick(1, 4, 5, 3)
			-- 5. Stick one piece to H

		tick(2, 4, 5, 4)
			-- 6. Tab connects to I

		multitick.execute
	end

	tick(i, j, x, y : INTEGER) is
	local
		t : ADD_TICK
		d : DIE
	do
		inspect pb.board.item(i, j, x, y)
		when 0 then
			!!t.make(pb, i, j, x, y)
			multitick.add_last(t)
		when -1 then
			io.put_string("warning: no solution%N")
			!!d.make(pb)
			multitick.add_last(d)
		else
		end
	end

	cross(i, j, x, y : INTEGER) is
	local
		ac : ADD_CROSS
	do
		!!ac.make(pb, i, j, x, y)
		do_initial_command(ac)
	end

	do_initial_command(c : COMMAND) is
	do
		if c.finished then
			io.put_string("warning: ")
			c.display
			io.put_string("already executed%N")
		elseif not c.can_execute then
			io.put_string("warning: ")
			c.display
			io.put_string(" => no solution%N")
		end
		c.execute
	end

	-- should go in lp_board?
	connect(i1, j1, x1, y1, i2, j2, x2, y2 : INTEGER) is
	local
		tl : TICK_LINK
		cl : CROSS_LINK
	do
		-- tick-link and cross-link both ways
		!!tl.make(i2, j2, x2, y2)
		pb.tick_observer.item(i1, j1, x1, y1).add_last(tl)
		!!cl.make(i2, j2, x2, y2)
		pb.cross_observer.item(i1, j1, x1, y1).add_last(cl)
		!!tl.make(i1, j1, x1, y1)
		pb.tick_observer.item(i2, j2, x2, y2).add_last(tl)
		!!cl.make(i1, j1, x1, y1)
		pb.cross_observer.item(i2, j2, x2, y2).add_last(cl)
	end
	
	larger_than(i1, x1, i2, x2, j : INTEGER) is
	local
		y : INTEGER
		larger : LARGER_THAN
		smaller : SMALLER_THAN
	do
		from y := 1
		until y > size
		loop
			-- for some reason, this problem can be solved even if
			-- one pair is commented out (eg use larger_than only)
			!!larger.make(i1, x1, i2, x2, j)
			pb.tick_observer.item(i1, j, x1, y).add_last(larger)
			pb.cross_observer.item(i1, j, x1, y).add_last(larger)
			!!smaller.make(i2, x2, i1, x1, j)
			pb.tick_observer.item(i2, j, x2, y).add_last(smaller)
			pb.cross_observer.item(i2, j, x2, y).add_last(smaller)
			y := y + 1
		end
	end

end

